package com.reactnativesample;

import android.content.Context;
import android.graphics.Color;
import android.util.Log;
import android.view.Choreographer;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentActivity;

import ad.helper.openbidding.BidmadCommon;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.common.MapBuilder;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.facebook.react.uimanager.SimpleViewManager;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.annotations.ReactProp;
import com.facebook.react.uimanager.annotations.ReactPropGroup;

import java.util.Map;

public class BidmadInterstitialViewManager extends SimpleViewManager<View> {
    public static final String CLASS_NAME = "BidmadInterstitial";

    public final int COMMAND_CREATE = 1;
    public final int COMMAND_SHOW = 2;
    public final int COMMAND_LOAD = 3;

    private int propMaxWidth;
    private int propWidth;
    private int propHeight;

    ReactApplicationContext reactContext;
    BidmadInterstitialAds view;

    public BidmadInterstitialViewManager(ReactApplicationContext reactContext) {
        this.reactContext = reactContext;
    }

    @NonNull
    @Override
    public String getName() {
        return CLASS_NAME;
    }

    @NonNull
    @Override
    protected View createViewInstance(@NonNull ThemedReactContext reactContext) {
        BidmadCommon.setDebugging(true);
        BidmadCommon.initializeSdk(reactContext);
        return new FrameLayout(reactContext);
    }

    @Nullable
    @Override
    public Map<String, Integer> getCommandsMap() {
        return MapBuilder.of(
                "create", COMMAND_CREATE,
                "show", COMMAND_SHOW,
                "load", COMMAND_LOAD
        );
    }

    @Override
    public void receiveCommand(@NonNull View root, String commandId, @Nullable ReadableArray args) {
        super.receiveCommand(root, commandId, args);

        int commandIdInt = Integer.parseInt(commandId);

        int reactNativeViewId = args.getInt(0);
        switch (commandIdInt) {
            case COMMAND_CREATE:
                String zoneId = args.getString(1);

//                view = new BidmadInterstitialAds(, zoneId);
                break;
            case COMMAND_SHOW:
                view.show();
                break;
            case COMMAND_LOAD:
                view.load();
                break;
            default: {}
        }
    }

    /**
     * Replace your React Native view with a custom fragment
     */
    public void createFragment(FrameLayout root, int reactNativeViewId, String zoneId) {
        ViewGroup parentView = (ViewGroup) root.findViewById(reactNativeViewId);

        final BidmadBannerFragment bannerFragment = new BidmadBannerFragment(zoneId, root, propWidth, propHeight);
        bannerFragment.setListener(new BidmadBannerListener() {
            @Override
            public void onLoadAd() {
                Context context = root.getContext();
                if (context instanceof ReactContext) {
                    ((ReactContext) context).getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                            .emit("onLoad_"+reactNativeViewId, zoneId);
//                    manuallyLayoutChildren(parentView, propMaxWidth, propWidth, propHeight);
                }
            }

            @Override
            public void onFailedAd() {
                Context context = root.getContext();
                if (context instanceof ReactContext) {
                    ((ReactContext) context).getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
                            .emit("onFail_"+reactNativeViewId, zoneId);
//                    manuallyLayoutChildren(parentView, 0, 0);
                }
            }
        });

        FragmentActivity activity = (FragmentActivity) reactContext.getCurrentActivity();
        activity.getSupportFragmentManager()
                .beginTransaction()
                .replace(reactNativeViewId, bannerFragment, String.valueOf(reactNativeViewId))
                .commit();
    }

    public void manuallyLayoutChildren(View view, int maxWidth, int width, int height) {
        view.measure(
                View.MeasureSpec.makeMeasureSpec(width, View.MeasureSpec.EXACTLY),
                View.MeasureSpec.makeMeasureSpec(height, View.MeasureSpec.EXACTLY));

        //전체 - 배너 크기 = 여백 / 2 = 좌/우여
        int leftPos = (maxWidth - width) / 2;
        view.layout(leftPos, view.getTop(), width, view.getBottom());
    }

    public void reloadBidmadBanner(int reactNativeViewId) {
        FragmentActivity activity = (FragmentActivity) reactContext.getCurrentActivity();
        BidmadBannerFragment bannerFragment = (BidmadBannerFragment) activity.getSupportFragmentManager().findFragmentById(reactNativeViewId);

        if(bannerFragment != null)
            bannerFragment.load();
    }
}
